<?php

namespace Lean\Api;

use Lean\Api\Attribute\ContextField;
use Notes\ObjectReflection;

class FormDescriptor
{
    use DescriptorTrait;

    public function getForms(...$forms) : array
    {
        $list = [];

        foreach($forms as $form) {
            $fields = [];
            $propertyHtml = "";
            $formName = is_object($form) ? $form::class : $form;
            $reflector = new ObjectReflection($formName);

            $context = $reflector->reflectClass()->getAttribute(ContextField::class);

            foreach($reflector->reflectProperties() as $property) {
                $field = $property->getAttribute(ContextField::class);

                if ($field) {
                    $types = $property->getTypes();
                    $fields[] = [
                        'name' => $property->name,
                        'description' => $field->object->description,
                        'type' => $field->object->type ?? implode(' | ', array_map(fn($e) => $e->type, $types)),
                        'allowNulls' => $property->allowsNull(),
                        'regexPattern' =>$field->object->regexFormat ?? "aucun",
                        'minLength' =>$field->object->minLength ?? "aucune",
                        'maxLength' =>$field->object->maxLength ?? "aucune",
                        'example' =>$field->object->example,
                    ];
                }
            }

            $list[$formName] = [
                'className' => $reflector->reflectClass()->getClassName(),
                'description' => $context->object->description ?? null,
                'fields' => $fields,
            ];
        }

        return $list;
    }
}