<?php

namespace Lean\Api\Middleware;

use Laminas\Diactoros\Response\JsonResponse;
use Lean\Factory\HttpFactory;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

class ApiRenderer implements MiddlewareInterface {

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
    {
        try { 
            $response = $handler->handle($request);
        }
        catch(\Exception $ex) {
            if ( ! getenv('DEBUG') ) {
                return HttpFactory::createJsonResponse([
                    'status' => 'failed',
                    'message' => $ex->getMessage(),
                    'ts' => time(),
                ], 400);
            }
            else {
                throw $ex;
            }
        }

        if ($response instanceof JsonResponse) {
            $payload = $response->getPayload();

            # For now, we match only response having a 'data' field
            if (isset($payload['data'])) {
                return HttpFactory::createJsonResponse([
                    'status' => 'success',
                    'ts' => time(),
                ] + $payload, 200);
            }
        }

        return $response;
    }
}
